# Authors:
# rafael@postgresql.org.es / http://www.postgresql.org.es/
#
# Copyright (c) 2014-2016 USIT-University of Oslo
#
# This file is part of Zabbix-CLI
# https://github.com/rafaelma/zabbix-cli
#
# Zabbix-CLI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zabbix-CLI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zabbix-CLI.  If not, see <http://www.gnu.org/licenses/>.
import collections
import configparser
import logging
import os
import sys

# Config file basename
CONFIG_FILENAME = 'zabbix-cli.conf'
CONFIG_FIXED_NAME = 'zabbix-cli.fixed.conf'

# Config file locations
CONFIG_DEFAULT_DIR = '/usr/share/zabbix-cli'
CONFIG_SYSTEM_DIR = '/etc/zabbix-cli'
CONFIG_USER_DIR = os.path.expanduser('~/.zabbix-cli')

# Any item will overwrite values from the previous
CONFIG_PRIORITY = tuple((
    os.path.join(d, f) for d, f in (
        (CONFIG_DEFAULT_DIR, CONFIG_FILENAME),
        (CONFIG_SYSTEM_DIR, CONFIG_FILENAME),
        (CONFIG_USER_DIR, CONFIG_FILENAME),
        (CONFIG_SYSTEM_DIR, CONFIG_FIXED_NAME),
        (CONFIG_DEFAULT_DIR, CONFIG_FIXED_NAME),
    )))

# Where custom configs should be put into the order
CONFIG_CUSTOM_GOES_AFTER = os.path.join(CONFIG_USER_DIR, CONFIG_FILENAME)

logger = logging.getLogger(__name__)


def get_priority(filename=None):
    """Get and ordered list of config file locations."""
    priority = list(CONFIG_PRIORITY)
    if filename:
        if CONFIG_CUSTOM_GOES_AFTER in priority:
            priority.insert(priority.index(CONFIG_CUSTOM_GOES_AFTER) + 1,
                            filename)
        else:
            priority.append(filename)
    return priority


def find_config(filename=None):
    """Find all available configuration files.

    :param filename: An optional user supplied file to throw into the mix
    """
    for filename in get_priority(filename):
        if os.path.isfile(filename):
            logger.debug('found config %r', filename)
            yield filename


class OptionDescriptor(object):
    """Descriptor to access ConfigParser settings as attributes."""

    # TODO: Add serialization, so that 'ON', 'OFF' -> boolean, etc...

    def __init__(self, section, option,
                 default=None, required=False, disable=False):
        self.section = section
        self.option = option
        self.default = default
        self.required = required
        self.disable = disable

    def __get__(self, obj, cls=None):
        if not obj:
            return self
        return obj.get(self.section, self.option)

    def __set__(self, obj, value):
        return obj.set(self.section, self.option, value)

    def __delete__(self, obj):
        return obj.set(self.section, self.option, self.default)

    def __repr__(self):
        return ('<{cls.__name__} {obj.section}.{obj.option}'
                ' default={obj.default!r}'
                ' required={obj.required!r}'
                ' disable={obj.disable!r}'
                '>').format(cls=type(self), obj=self)


class OptionRegister(collections.abc.Mapping):
    """A registry of ConfigParser sections, options and default values."""

    def __init__(self):
        self._settings = collections.OrderedDict()

    def __len__(self):
        return len(self._settings)

    def __iter__(self):
        return iter(self._settings)

    def __getitem__(self, option_tuple):
        return self._settings[option_tuple]

    def add(self, section, option, *args, **kwargs):
        self._settings[(section, option)] = OptionDescriptor(section, option,
                                                             *args, **kwargs)
        return self._settings[(section, option)]

    @property
    def sections(self):
        def _get():
            seen = set()
            for section, option in self:
                if section not in seen:
                    seen.add(section)
                    yield section
        return tuple(_get())

    def initialize(self, obj):
        seen = set()
        for section, option in self:
            if section not in seen:
                seen.add(section)
                obj.add_section(section)
            obj.set(section, option, self[(section, option)].default)


class Configuration(configparser.RawConfigParser, object):
    """A custom ConfigParser object with zabbix-cli settings."""

    _registry = OptionRegister()

    zabbix_api_url = _registry.add(
        'zabbix_api', 'zabbix_api_url',
        required=True)

    cert_verify = _registry.add(
        'zabbix_api', 'cert_verify',
        default='ON')

    system_id = _registry.add(
        'zabbix_config', 'system_id',
        default='zabbix-ID')

    default_hostgroup = _registry.add(
        'zabbix_config', 'default_hostgroup',
        default='All-hosts')

    default_admin_usergroup = _registry.add(
        'zabbix_config', 'default_admin_usergroup',
        default='All-root')

    default_create_user_usergroup = _registry.add(
        'zabbix_config', 'default_create_user_usergroup',
        default='All-users')

    default_notification_users_usergroup = _registry.add(
        'zabbix_config', 'default_notification_users_usergroup',
        default='All-notification-users')

    default_directory_exports = _registry.add(
        'zabbix_config', 'default_directory_exports',
        default=os.path.expanduser('~/zabbix_exports'))

    default_export_format = _registry.add(
        'zabbix_config', 'default_export_format',
        default='XML',
        # We deactivate this until https://support.zabbix.com/browse/ZBX-10607
        # gets fixed.  We use XML as the export format.
        disable=True)

    include_timestamp_export_filename = _registry.add(
        'zabbix_config', 'include_timestamp_export_filename',
        default='ON'
    )

    use_colors = _registry.add(
        'zabbix_config', 'use_colors',
        default='ON')

    use_auth_token_file = _registry.add(
        'zabbix_config', 'use_auth_token_file',
        default='OFF')

    use_paging = _registry.add(
        'zabbix_config', 'use_paging',
        default='OFF')

    logging = _registry.add(
        'logging', 'logging',
        default='OFF')

    log_level = _registry.add(
        'logging', 'log_level',
        default='ERROR')

    log_file = _registry.add(
        'logging', 'log_file',
        default='/var/log/zabbix-cli/zabbix-cli.log')

    def __init__(self):
        super(Configuration, self).__init__()
        self._registry.initialize(self)
        self.loaded_files = []

    def set(self, section, option, value):
        descriptor = self._registry[(section, option)]
        if descriptor.disable and value != descriptor.default:
            logger.warning('setting %s.%s is disabled, setting to %r',
                           section, option, descriptor.default)
            value = descriptor.default
        return super(Configuration, self).set(section, option, value)

    def read(self, filenames):
        files_read = super(Configuration, self).read(filenames)
        for filename in files_read:
            if filename in self.loaded_files:
                self.loaded_files.remove(filename)
            self.loaded_files.append(filename)
        return files_read

    def readfp(self, fp, filename=None):
        try:
            add_filename = filename or fp.name
        except AttributeError:
            pass
        retval = super(Configuration, self).readfp(fp, filename=filename)
        if add_filename:
            self.loaded_files.append(add_filename)
        return retval  # should be None

    def iter_descriptors(self):
        return iter(self._registry.values())

    def iter_required(self):
        for descriptor in self.iter_descriptors():
            if descriptor.required:
                yield descriptor

    def iter_missing(self):
        for descriptor in self.iter_required():
            value = self.get(descriptor.section, descriptor.option)
            if value == descriptor.default:
                yield descriptor


def get_config(filename=None):
    config = Configuration()
    for filename in find_config(filename):
        logger.debug('loading config %r', filename)
        config.read(filename)
    return config


def validate_config(config):
    missing = ['{0.section}.{0.option}'.format(d)
               for d in config.iter_missing()]
    if missing:
        raise ValueError("Missing settings: " + ', '.join(missing))


#
# python -m zabbix_cli.config
#

def main(inargs=None):
    import argparse

    class Actions(object):
        """Subparser to function map."""

        def __init__(self):
            self.funcmap = dict()

        def __getitem__(self, key):
            return self.funcmap[key]

        def __call__(self, subparser):
            def wrapper(func):
                key = subparser.prog.split(' ')[-1]
                self.funcmap[key] = func
                return func
            return wrapper

    parser = argparse.ArgumentParser(description='write default config')
    commands = parser.add_subparsers(title='commands', dest='command')
    actions = Actions()

    #
    # defaults [filename]
    #
    defaults_cmd = commands.add_parser('defaults')
    defaults_cmd.add_argument(
        'output',
        type=argparse.FileType('w'),
        nargs='?',
        default='-',
        metavar='FILE',
        help='Write an example config to %(metavar)s (default: stdout)')

    @actions(defaults_cmd)
    def write_default_config(args):
        config = Configuration()
        config.write(args.output)
        args.output.flush()
        if args.output not in (sys.stdout, sys.stderr):
            args.output.close()

    #
    # show
    #
    show_cmd = commands.add_parser('show')
    show_cmd.add_argument(
        '-c', '--config',
        default=None,
        metavar='FILE',
        help='Use config from %(metavar)s')
    show_cmd.add_argument(
        '-v', '--validate',
        action='store_true',
        default=False,
        help='validate config')

    @actions(show_cmd)
    def show_config(args):
        config = get_config(args.config)
        if args.validate:
            validate_config(config)
        config.write(sys.stdout)
        sys.stdout.flush()

    args = parser.parse_args()
    actions[args.command](args)


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    main()



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

import os
import sys
import re

# Add the parent directory on PYTHONPATH.
sys.path.insert(0, os.path.abspath('..'))


# -- Project information -----------------------------------------------------

project = 'zabbix-cli'
copyright = '2023, University of Oslo'
author = 'University of Oslo'

# The full version, including alpha/beta/rc tags
release = '2.3.2'


# -- General configuration ---------------------------------------------------

root_doc = "manual"

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'alabaster'

html_theme_options = {
    "nosidebar": True,
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

html_css_files = [
    'css/manual.css',
]

html_js_files = [
    'js/manual.js',
]

# -- Extension configuration -------------------------------------------------

# Select only members with "do_" prefix.
def autodoc_skip_member_handler(app, what, name, obj, skip, options):
    return not name.startswith("do_")

# Hide the Python function signature description.
def autodoc_process_signature_handler(app, what, name, obj, options, signature, return_annotation):
    return "", None

def autodoc_process_docstring_handler(app, what, name, obj, options, lines):
    """
    Remove all DESCRIPTION.
    Change all COMMAND: to be rendered as code blocks.
    Sections don't work in docstrings, so change them to .. rubric::.
    """
    new_lines = []
    for idx in range(len(lines)):
        line = lines[idx]
        desc = re.match("^DESCRIPTION:?(.+)?", line)
        if desc:
            if desc[1] is not None:
                # Take 'foo' from 'DESCRIPTION: foo'.
                new_lines.append(desc[1])
            continue
        command = re.match("^COMMAND:(.+)?$", line)
        if command:
            new_lines.append("::")
            new_lines.append("")
            # Handle 'COMMAND: foo [bar]'.
            if command[1] is not None:
                new_lines.append("  " + command[1])
            # Modify the next items ahead of time.  We need to figure out
            # how many arguments (if any), and indent each line.
            lines_to_indent = 0
            for i in range(idx + 1, len(lines)):
                # Stop at the first empty line
                next_line = lines[i]
                if len(next_line) > 0:
                    lines_to_indent += 1
                else:
                    break
            for i in range(lines_to_indent):
                lines[idx+1+i] = "  " + lines[idx+1+i]
            continue
        section = re.match("^-+$", line)
        if section:
            # For sections, remove the --- markers and instead add
            # .. rubric:: to the previous line, plus a blank line.
            old = new_lines[-1]
            new = ".. rubric:: " + old
            new_lines[-1] = new
            new_lines.append("")
            continue
        new_lines.append(line)

    lines.clear()
    lines += new_lines

def setup(app):
    app.connect('autodoc-process-docstring', autodoc_process_docstring_handler)
    app.connect('autodoc-process-signature', autodoc_process_signature_handler)
    app.connect('autodoc-skip-member', autodoc_skip_member_handler)



"""All functions in this module extend or simplifies common API tasks."""


def update_usergroup(zapi, usrgrpid, rights=None, userids=None):
    """
    Merge update a usergroup.

    Updating usergroups without replacing current state (i.e. merge update) is hard.
    This function simplifies the process.

    The rights and userids provided are merged into the usergroup.
    """
    usrgrpid = str(usrgrpid)  # Make sure this number is a string
    usergroup = zapi.usergroup.get(filter={"usrgrpid": usrgrpid}, selectRights=["permission", "id"], selectUsers=["userid"])[0]

    if rights:
        # Get the current rights with ids from new rights filtered
        new_rights = [current_right for current_right in usergroup["rights"] if current_right["id"] not in [right["id"] for right in rights]]

        new_rights.extend(rights)

        return zapi.usergroup.update(usrgrpid=usrgrpid, rights=new_rights)

    if userids:
        current_userids = [user["userid"] for user in usergroup["users"]]

        # Make sure we only have unique ids
        new_userids = list(set(current_userids + userids))

        return zapi.usergroup.update(usrgrpid=usrgrpid, userids=new_userids)

    return None



import configparser
import unittest

import zabbix_cli.config


# TODO: Switch from RawConfigParser to ConfigParser here and config.py
class MockConfig(configparser.RawConfigParser, object):

    def __init__(self):
        super(MockConfig, self).__init__({'option_a': 'a', 'option_b': 'b'})
        self.add_section('foo')
        self.add_section('bar')


class TestOptionDescriptor(unittest.TestCase):

    class MockConfigWithDescriptors(MockConfig):
        foo_a = zabbix_cli.config.OptionDescriptor('foo', 'option_a')
        bar_b = zabbix_cli.config.OptionDescriptor('bar', 'option_b')

    def _make_config(self):
        return self.MockConfigWithDescriptors()

    def test_descriptor_cls(self):
        cls = self.MockConfigWithDescriptors
        desc = cls.foo_a
        self.assertTrue(isinstance(desc, zabbix_cli.config.OptionDescriptor))

    def test_descriptor_get(self):
        config = self._make_config()
        self.assertEqual(config.get('foo', 'option_a'),
                         getattr(config, 'foo_a'))

    def test_descriptor_set(self):
        config = self._make_config()
        config.foo_a = 'foo'
        self.assertEqual('foo', config.foo_a)
        self.assertEqual(config.get('foo', 'option_a'),
                         config.foo_a)

    def test_descriptor_del(self):
        config = self._make_config()
        config.foo_a = 'foo'
        del config.foo_a
        self.assertFalse('foo' == config.foo_a)
        self.assertEqual(config.get('foo', 'option_a'),
                         config.foo_a)


class TestOptionRegister(unittest.TestCase):

    def _make_register(self):
        return zabbix_cli.config.OptionRegister()

    def test_len(self):
        register = self._make_register()
        self.assertEqual(0, len(register))

        register.add('foo', 'bar')
        self.assertEqual(1, len(register))

        register.add('foo', 'baz')
        self.assertEqual(2, len(register))

    def test_iter(self):
        items = [('foo', 'bar'), ('foo', 'baz')]
        register = self._make_register()

        for section, option in items:
            register.add(section, option)

        for i, item in enumerate(register):
            self.assertEqual(items[i], item)

    def test_getitem(self):
        items = [('foo', 'bar'), ('foo', 'baz')]
        register = self._make_register()

        comparison = []
        for section, option in items:
            comparison.append(register.add(section, option))

        for i, item in enumerate(items):
            gotten = register[item]
            self.assertEqual(comparison[i], gotten)

    def test_sections(self):
        items = [('foo', 'bar'), ('foo', 'baz'), ('bar', 'foo')]
        expect = set(t[0] for t in items)
        register = self._make_register()

        for section, option in items:
            register.add(section, option)

        self.assertEqual(len(expect), len(register.sections))
        self.assertEqual(set(expect), set(register.sections))

    def test_initialize(self):
        items = [('foo', 'bar'), ('foo', 'baz'), ('bar', 'foo')]
        sections = set(t[0] for t in items)
        register = self._make_register()

        for section, option in items:
            register.add(section, option)

        config = configparser.RawConfigParser()
        register.initialize(config)

        for section in sections:
            self.assertTrue(config.has_section(section))

        for section, option in items:
            self.assertTrue(config.has_option(section, option))



import pytest
from packaging.version import Version

from zabbix_cli.pyzabbix import user_param_from_version


@pytest.mark.parametrize(
        "version, expect",
        [
            (Version("7.0.0"), "username"),
            (Version("6.0.0"), "username"),
            (Version("6.2.0"), "username"),
            (Version("6.4.0"), "username"),
            (Version("5.4.0"), "username"),
            (Version("5.4.1"), "username"),
            (Version("5.3.9"), "user"),
            (Version("5.2.0"), "user"),
            (Version("5.2"), "user"),
            (Version("5.0"), "user"),
            (Version("4.0"), "user"),
            (Version("2.0"), "user"),
        ],
)
def test_user_param_from_version(version: Version, expect: str):
    assert user_param_from_version(version) == expect




import logging
import unittest

import zabbix_cli.logs


logger = logging.getLogger(__name__)


class CollectHandler(logging.Handler):

    def __init__(self):
        super(CollectHandler, self).__init__(logging.NOTSET)
        self.records = []

    def emit(self, record):
        print('emit', repr(record))
        self.records.append(record)


class TestSafeFormatting(unittest.TestCase):

    def _make_log_record(self, msg, *args):
        # locals for readability
        record_logger = 'example-logger-name'
        record_level = logging.ERROR
        record_pathname = __file__
        record_lineno = 1
        record_exc_info = None
        return logging.LogRecord(record_logger, record_level, record_pathname,
                                 record_lineno, msg, args, record_exc_info)

    def _make_safe_record(self, msg, *args):
        return zabbix_cli.logs.SafeRecord(self._make_log_record(msg, *args))

    def test_safe_record(self):
        self._make_safe_record("foo")
        self.assertTrue(True)  # reached

    def test_safe_record_basic_message(self):
        message = "foo"
        record = self._make_safe_record(message)
        self.assertEqual(message, record.getMessage())

    def test_safe_record_formatted_message(self):
        expect = "foo 01 02"
        record = self._make_safe_record("foo %02d %02d", 1, 2)
        self.assertEqual(expect, record.getMessage())

    def test_safe_record_attr(self):
        msg = "foo",
        args = (1, 2)
        record = self._make_safe_record(msg, *args)
        self.assertEqual(msg, record.msg)
        self.assertEqual(args, record.args)

    def test_safe_record_missing_dict(self):
        fmt = "%(msg)s-%(this_probably_does_not_exist)s"
        record = self._make_safe_record("foo")
        expect = "foo-%s" % (None, )
        result = fmt % record.__dict__
        self.assertEqual(expect, result)

    def test_safe_formatter(self):
        fmt = "%(name)s - %(something)s - %(msg)s"
        formatter = zabbix_cli.logs.SafeFormatter(fmt)
        record = self._make_log_record("foo")

        expect = fmt % {'name': record.name,
                        'something': None,
                        'msg': record.msg}
        self.assertEqual(expect, formatter.format(record))


class TestLoggingContext(unittest.TestCase):

    def _get_log_record(self):
        return type('mock_LogRecord', (object, ), {})()

    def _get_logger(self, test_name):
        handler = CollectHandler()
        test_logger = logger.getChild(test_name)
        test_logger.addHandler(handler)
        test_logger.propagate = False
        test_logger.disabled = False
        test_logger.setLevel(logging.NOTSET + 1)
        return test_logger, handler.records

    def test_context_filter_returns_true(self):
        f = zabbix_cli.logs.ContextFilter('foo', 'bar')
        r = self._get_log_record()
        self.assertTrue(f.filter(r))

    def test_context_filter_sets_field(self):
        f = zabbix_cli.logs.ContextFilter('foo', 'bar')
        r = self._get_log_record()
        f.filter(r)
        self.assertEqual('bar', r.foo)

    def test_context_filter_resets_field(self):
        f1 = zabbix_cli.logs.ContextFilter('foo', 'bar')
        f2 = zabbix_cli.logs.ContextFilter('foo', 'baz')
        r = self._get_log_record()
        f1.filter(r)
        f2.filter(r)
        self.assertEqual('baz', r.foo)

    def test_log_context_no_context(self):
        test_logger, records = self._get_logger('test_log_context_no_context')
        fmt = "%(no_field)s"
        formatter = zabbix_cli.logs.SafeFormatter(fmt)
        test_logger.info("foo")
        self.assertEqual(1, len(records))
        self.assertEqual(fmt % {'no_field': None},
                         formatter.format(records[0]))

    def test_log_context(self):
        test_logger, records = self._get_logger('test_log_context')
        fmt = "%(no_field)s"
        formatter = zabbix_cli.logs.SafeFormatter(fmt)
        with zabbix_cli.logs.LogContext(test_logger, no_field='foo'):
            test_logger.info("foo")
        self.assertEqual(1, len(records))
        self.assertEqual("foo", formatter.format(records[0]))

    def test_log_context_nested(self):
        test_logger, records = self._get_logger('test_log_context_nested')
        fmt = "%(no_field)s"
        formatter = zabbix_cli.logs.SafeFormatter(fmt)
        with zabbix_cli.logs.LogContext(test_logger, no_field='foo'):
            with zabbix_cli.logs.LogContext(test_logger, no_field='bar'):
                test_logger.info("nested")
            test_logger.info("first")
        test_logger.info("no context")

        self.assertEqual(3, len(records))
        # first record was nested in two LogContexts
        self.assertEqual("bar", formatter.format(records[0]))
        # second record was nested in one LogContext
        self.assertEqual("foo", formatter.format(records[1]))
        # last record was logged after contexts
        self.assertEqual(fmt % {'no_field': None},
                         formatter.format(records[2]))



"""Utility functions."""


def get_ack_status(code):
    """Get ack status from code."""
    ack_status = {0: "No", 1: "Yes"}

    if code in ack_status:
        return ack_status[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_event_status(code):
    """Get event status from code."""
    event_status = {0: "OK", 1: "Problem"}

    if code in event_status:
        return event_status[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_trigger_severity(code):
    """Get trigger severity from code."""
    trigger_severity = {0: "Not classified", 1: "Information", 2: "Warning", 3: "Average", 4: "High", 5: "Disaster"}

    if code in trigger_severity:
        return trigger_severity[code]

    return "Unknown ({})".format(str(code))


def get_trigger_status(code):
    """Get trigger status from code."""
    trigger_status = {0: "Enable", 1: "Disable"}

    if code in trigger_status:
        return trigger_status[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_maintenance_status(code):
    """Get maintenance status from code."""
    maintenance_status = {0: "No maintenance", 1: "In progress"}

    if code in maintenance_status:
        return maintenance_status[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_monitoring_status(code):
    """Get monitoring status from code."""
    monitoring_status = {0: "Monitored", 1: "Not monitored"}

    if code in monitoring_status:
        return monitoring_status[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_zabbix_agent_status(code):
    """Get zabbix agent status from code."""
    zabbix_agent_status = {1: "Available", 2: "Unavailable"}

    if code in zabbix_agent_status:
        return zabbix_agent_status[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_gui_access(code):
    """Get GUI access from code."""
    gui_access = {0: "System default", 1: "Internal", 2: "Disable"}

    if code in gui_access:
        return gui_access[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_usergroup_status(code):
    """Get usergroup status from code."""
    usergroup_status = {0: "Enable", 1: "Disable"}

    if code in usergroup_status:
        return usergroup_status[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_hostgroup_flag(code):
    """Get hostgroup flag from code."""
    hostgroup_flag = {0: "Plain", 4: "Discover"}

    if code in hostgroup_flag:
        return hostgroup_flag[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_hostgroup_type(code):
    """Get hostgroup type from code."""
    hostgroup_type = {0: "Not internal", 1: "Internal"}

    if code in hostgroup_type:
        return hostgroup_type[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_user_type(code):
    """Get user type from code."""
    user_type = {1: "User", 2: "Admin", 3: "Super admin", 4: "Guest"}

    if code in user_type:
        return user_type[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_maintenance_type(code):
    """Get maintenance type from code."""
    maintenance_type = {0: "With DC", 1: "Without DC"}

    if code in maintenance_type:
        return maintenance_type[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_maintenance_period_type(code):
    """Get maintenance period type from code."""
    maintenance_period_type = {0: "One time", 2: "Daily", 3: "Weekly", 4: "Monthly"}

    if code in maintenance_period_type:
        return maintenance_period_type[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_autologin_type(code):
    """Get autologin type from code."""
    autologin_type = {0: "Disable", 1: "Enable"}

    if code in autologin_type:
        return autologin_type[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))


def get_permission(code):
    """Get permission."""
    permission = {
        0: "deny",
        2: "ro",
        3: "rw"
    }

    return permission.get(code, None)


def get_permission_code(permission):
    """Get permission code."""
    permission_code = {"deny": 0, "ro": 2, "rw": 3}

    if permission in permission_code:
        return permission_code[permission]

    return 0


def get_item_type(code):
    """Get item type from code."""
    item_type = {0: "Zabbix agent",
                 1: "SNMPv1 agent",
                 2: "Zabbix trapper",
                 3: "simple check",
                 4: "SNMPv2 agent",
                 5: "Zabbix internal",
                 6: "SNMPv3 agent",
                 7: "Zabbix agent (active)",
                 8: "Zabbix aggregate",
                 9: "web item",
                 10: "external check",
                 11: "database monitor",
                 12: "IPMI agent",
                 13: "SSH agent",
                 14: "TELNET agent",
                 15: "calculated",
                 16: "JMX agent",
                 17: "SNMP trap"}

    if code in item_type:
        return item_type[code] + " (" + str(code) + ")"

    return "Unknown ({})".format(str(code))



"""Compatibility functions for different Zabbix versions."""

from packaging.version import Version

# TODO (pederhan): rewrite these functions as some sort of declarative data
# structure that can be used to determine correct parameters based on version
# if we end up with a lot of these functions. For now, this is fine.
# OR we could turn it into a mixin class?

# Compatibility methods for Zabbix API objects properties and method parameter names (same thing)
# Returns the appropriate property name for the given Zabbix version.
# 
# FORMAT: <object>_<property>
# EXAMPLE: user_name() (User object, name property)
#
# DEV NOTE: All functions follow the same pattern: 
# Early return if the version is older than the version where the property 
# was deprecated, otherwise return the new property name as the default.


def host_proxyid(version: Version) -> str:
    # https://support.zabbix.com/browse/ZBXNEXT-8500
    # https://www.zabbix.com/documentation/7.0/en/manual/api/changes#host
    if version.release < (7, 0, 0):
        return "proxy_hostid"
    return "proxyid" 


def login_user_name(version: Version) -> str:
    # https://support.zabbix.com/browse/ZBXNEXT-8085
    # Deprecated in 5.4.0, removed in 6.4.0
    # login uses different parameter names than the User object before 6.4
    # From 6.4 and onwards, login and user.<method> use the same parameter names
    # See: user_name
    if version.release < (5, 4, 0):
        return "user"
    return "username"


def proxy_name(version: Version) -> str:
    # https://support.zabbix.com/browse/ZBXNEXT-8500
    # https://www.zabbix.com/documentation/7.0/en/manual/api/changes#proxy
    if version.release < (7, 0, 0):
        return "host"
    return "name"


def user_name(version: Version) -> str:
    # https://support.zabbix.com/browse/ZBXNEXT-8085
    # Deprecated in 5.4, removed in 6.4
    # However, historically we have used "alias" as the parameter name
    # pre-6.0.0, so we maintain that behavior here
    if version.release < (6, 0, 0):
        return "alias"
    return "username"



# Authors:
# rafael@postgresql.org.es / http://www.postgresql.org.es/
#
# Copyright (c) 2014-2015 USIT-University of Oslo
#
# This file is part of Zabbix-CLI
# https://github.com/rafaelma/zabbix-cli
#
# Zabbix-CLI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zabbix-CLI is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zabbix-CLI.  If not, see <http://www.gnu.org/licenses/>.
import collections
import logging
import sys

logger = logging.getLogger(__name__)

DEFAULT_FORMAT = " ".join((
    "%(asctime)s",
    "[%(name)s][%(user)s][%(process)d][%(levelname)s]:",
    "%(message)s",
))


class ContextFilter(logging.Filter):
    """Log filter that adds a static field to a record."""

    def __init__(self, field, value):
        self.field = field
        self.value = value

    def filter(self, record):
        setattr(record, self.field, self.value)
        return True


class LogContext(object):
    """A context that adds ContextFilters to a logger."""

    def __init__(self, logger, **context):
        self.logger = logger
        self.filters = [ContextFilter(k, context[k]) for k in context]

    def __enter__(self):
        for f in self.filters:
            self.logger.addFilter(f)
        return self.logger

    def __exit__(self, *args, **kwargs):
        for f in self.filters:
            self.logger.removeFilter(f)


class SafeRecord(logging.LogRecord, object):
    """A LogRecord wrapper that returns None for unset fields."""

    def __init__(self, record):
        self.__dict__ = collections.defaultdict(lambda: None, record.__dict__)


class SafeFormatter(logging.Formatter):
    """A Formatter that use SafeRecord to avoid failure."""

    def format(self, record):
        record = SafeRecord(record)
        return super(SafeFormatter, self).format(record)


def get_log_level(level):
    if level and level.isdigit():
        return int(level)
    elif level:
        # Given a name, getLevelName returns the int level
        return logging.getLevelName(level.upper())
    else:
        return logging.NOTSET


def configure_logging(config):
    """Configure the root logger."""
    enable = config.logging == 'ON'
    level = get_log_level(config.log_level)
    filename = config.log_file

    if enable and filename:
        # log to given filename
        handler = logging.FileHandler(filename)
    elif enable:
        # log to stderr
        handler = logging.StreamHandler(sys.stderr)
    else:
        # disable logging
        handler = logging.NullHandler()

    handler.setFormatter(SafeFormatter(fmt=DEFAULT_FORMAT))
    root = logging.getLogger()
    root.addHandler(handler)
    root.setLevel(level)


#
# python -m zabbix_cli.logs
#

def main(inargs=None):
    import argparse
    from zabbix_cli.config import get_config

    parser = argparse.ArgumentParser('test log settings')
    parser.add_argument(
        '-c', '--config',
        default=None)
    parser.add_argument(
        '--level',
        dest='log_level',
        default=None,
        help='override %(dest)s from config')
    parser.add_argument(
        '--file',
        dest='log_file',
        default=None,
        help='override %(dest)s from config')
    parser.add_argument(
        '--enable',
        dest='logging',
        choices=('ON', 'OFF'),
        default=None,
        help='override %(dest)s from config')

    args = parser.parse_args(inargs)
    config = get_config(args.config)

    for attr in ('logging', 'log_level', 'log_file'):
        value = getattr(args, attr)
        if value is not None:
            setattr(config, attr, value)

    configure_logging(config)

    logger.debug('a debug message')
    logger.info('an info message')
    logger.warning('a warn message')
    logger.error('an error message')
    try:
        this_name_is_not_in_scope  # noqa: F821
    except NameError:
        logger.error('an error message with traceback', exc_info=True)

    logger.debug("Message without user context")
    with LogContext(logger, user='user1'):
        logger.debug("Message with context user=user1")
        with LogContext(logger, user='user2'):
            logger.debug("Message with nested context user=user2")
    logger.debug("Message after context")

    logger.info('done')


if __name__ == '__main__':
    main()
